#!/usr/bin/env python

# @uthor Benoit Neviani (benoit.neviani@viavisolutions.com)
# version v.1.0

# import
import requests
import json
import os
import stat
import socket
import argparse
import logging
import datetime
import uuid
import time
import urllib3
import csv

# argurments check
parser = argparse.ArgumentParser()
parser.add_argument("-v","--version", action='version', version='%(prog)s : version v.1.0')
parser.add_argument("-define", action='store_const', const='define', dest='funct', help='define variables', metavar='')
parser.add_argument("-import", action='store_const', const='import', dest='funct', help='import subnets from CSV/TXT file [defined in -define function]')
parser.add_argument("--debug", help='debug level', action='store_const', const='debug', metavar='')
whattodo = parser.parse_args()
args, leftovers = parser.parse_known_args()

if args.debug is not None :
     logging.basicConfig(level=logging.DEBUG)
     logging.getLogger("requests").setLevel(logging.DEBUG)
else :
     logging.basicConfig(level=logging.INFO)
     logging.getLogger("requests").setLevel(logging.WARNING)
logger = logging.getLogger()
formatter = logging.Formatter('%(asctime)s :: %(levelname)s :: %(message)s')
handler = logging.FileHandler('apex.log')
logger.addHandler(handler)
handler.setFormatter(formatter)

# functions
## variables
def writevar(name,value):
    write = {name:value}
    try:
        with open('var.json', 'a') as var:
            json.dump(write, var)
            var.write(os.linesep)
            var.close()
    except IOError as e:
            logger.info("impossible to write var.json, please check permission/disk space...")
            exit(1)
def readvar(name,file):
    data = []
    with open(file, 'r') as var:
        for line in var:
            data.append(json.loads(line))
        for key in data:
            if name in key:
                return key[name]

def defvariables():
    csvpath = input("Full path of CSV file: ")
    while csvpath is '':
        csvpath = input("Please enter Full path of CSV file: ")
    writevar("csvpath",csvpath)
    writevar("baseURL","https://")
    writevar("sessionURL","/oa/api/session")
    writevar("businessgroupURL","/oa/api/businessGroups")
    ipAddress = input("Apex IP Address/DNS name: ")
    while ipAddress is '':
        ipAddress = input("Please enter Apex IP Address/DNS name: ")
    writevar("ipAddress",ipAddress)
    username = input("Apex User: ")
    while username is '':
        username = input("Please enter Apex User: ")
    writevar("username",username)
    password = input("Apex Password: ")
    while password is '':
        password = input("Please enter Apex Password: ")
    writevar("password",password)
    bg = input("Apex EXISTING Business Group where imported Business Groups will be derived from: ")
    while bg is '':
        bg = input("Please enter Apex EXISTING Business Group: ")
    writevar("bg",bg)
    userid = getUserId()
    writevar("UserId",userid)
    logger.info("var.json created successfully...")

def testvariables():
    sessionCookie = login()
    try:
        with open(readvar("csvpath","var.json"), 'a') as csvfile:
            csvfile.close()
    except IOError as e:
            logger.info("impossible to find" + readvar("csvpath","var.json") +" , please check file path...")
            exit(1)
    if not getBusinessGroupId(sessionCookie):
        logger.info("There is no Business Group named " + readvar("bg","var.json") + " in Apex please check the Business Group name in Apex !")
        os.remove('var.json')
        exit(1)
    writevar("BusinessGroupId",getBusinessGroupId(sessionCookie))
    logout(sessionCookie)

## http methods
def getRequest(sessionCookie, url):
    Cookie = "SessionApexId="+sessionCookie
    headers = {'Content-Type':'Application/json','Accept':'Application/json','Cookie':Cookie}
    request = requests.get(url, headers = headers, verify = False)
    logger.debug("HTTP GET is " + str(request) + "\n")
    return request

def postRequest(sessionCookie, url, data):
    Cookie = "SessionApexId="+sessionCookie
    headers = {'Content-Type':'Application/json','Accept':'Application/json','Cookie':Cookie}
    request = requests.post(url, headers = headers, data = data, verify = False)
    logger.debug("HTTP POST is " + str(request) + "\n")
    return request

def putRequest(sessionCookie, url, data):
    Cookie = "SessionApexId="+sessionCookie
    headers = {'Content-Type':'Application/json','Accept':'Application/json','Cookie':Cookie}
    request = requests.put(url, headers = headers, data = data, verify = False)
    logger.debug("HTTP PUT is " + str(request) + "\n")
    return request

def deleteRequest(sessionCookie, url):
    Cookie = "SessionApexId="+sessionCookie
    headers = {'Content-Type':'Application/json','Accept':'Application/json','Cookie':Cookie}
    request = requests.delete(url, headers = headers, verify = False)
    logger.debug("HTTP DELETE is " + str(request) + "\n")
    return request

## sessions management
def login():
    url = readvar("baseURL","var.json") + readvar("ipAddress","var.json") + readvar("sessionURL","var.json")
    data = '{"session": {"username": "'+readvar("username","var.json")+'","password": "'+readvar("password","var.json")+'"}}'
    logger.debug("Apex data request is " + str(data) + "\n")
    response = postRequest('nothing', url, data)
    logger.debug("Apex login data response is " + str(response.text) + "\n")
    if (response.ok):
        logger.info("Login successfully")
        cookie = response.cookies['SessionApexId']
        logger.debug("SessionApexId is " + str(cookie) + "\n")
        return cookie
    else:
        logger.info("Login failed")
        exit(1)

def logout(sessionCookie):
    url = readvar("baseURL","var.json") + readvar("ipAddress","var.json") + readvar("sessionURL","var.json")
    response = deleteRequest(sessionCookie, url)
    logger.debug("Apex logout data response is " + str(response.text) + "\n")
    if (response.ok):
            logger.info("Logout successfully")
    else:
            logger.info("Logout Failed")
            exit(1)

## monitoring
def monitoring():
    logger.info("*****Start script*****")
    ping = os.system("ping -c 1 -w2 " + readvar("ipAddress","var.json") + " 2>&1 >/dev/null")
    if ping == 0:
        logger.info("Apex is up")
    else:
        logger.info("Apex is down")
        exit(1)
    s = socket.socket()
    try:
        s.connect((readvar("ipAddress","var.json"), 443))
    except Exception as connection:
        logger.info("Apex connection refused")
        exit(1)
    finally:
        s.close()

## retrieve Ids
def getBusinessGroupId(sessionCookie):
    url = readvar("baseURL","var.json") + readvar("ipAddress","var.json") + readvar("businessgroupURL","var.json")
    response = getRequest(sessionCookie, url)
    json_response = json.loads(response.text)
    logger.debug("Apex business group data response is " + str(response.text) + "\n")
    bg_id = ''
    for key in json_response['data']['businessgroups']:
        if key['name'] == readvar("bg","var.json"):
            bg_id = key['id']
            logger.debug("Apex business group " + readvar("bg","var.json") + " id is " + str(bg_id) + "\n")
            return bg_id
    if bg_id == '':
        logger.info("Business Group id not found")

def getUserId():
    url = readvar("baseURL","var.json") + readvar("ipAddress","var.json") + readvar("sessionURL","var.json")
    data = '{"session": {"username": "'+readvar("username","var.json")+'","password": "'+readvar("password","var.json")+'"}}'
    logger.debug("Apex data request is " + str(data) + "\n")
    response = postRequest('nothing', url, data)
    json_response = json.loads(response.text)
    logger.debug("Apex login data response is " + str(response.text) + "\n")
    if (response.ok):
        logger.info("Login successfully")
        sessionCookie = response.cookies['SessionApexId']
        userid = json_response['data']['LoginSession']['user']['id']
        logger.debug("UserId is " + str(userid) + "\n")
        deleteRequest(sessionCookie, url)
        return userid
    else:
        logger.info("Login failed")
        exit(1)

## create functions
def create_bg(sessionCookie, bgname, bgfilter):
    url = readvar("baseURL","var.json") + readvar("ipAddress","var.json") + readvar("businessgroupURL","var.json")
    data = '{"businessgroup": {"name": "'+bgname+'","description": "Business Group Generated by script","flags": {"enabled": true, "visible": true} ,"filter": "ip '+bgfilter+'" ,"children": [{"childId": "'+readvar("BusinessGroupId","var.json")+'"}]}}'
    logger.debug("Apex data request data is " + str(data) + "\n")
    response = postRequest(sessionCookie, url, data)
    json_response = json.loads(response.text)
    logger.debug("Apex bg data response is " + str(response.text) + "\n")

## csv
def readcsv(file):
    with open(file, 'r') as csvfile:
         reader = csv.reader(csvfile, delimiter=',')
         for row in reader:
            bgname = row[0]
            bgfilter = row[1]
            create_bg(sessionCookie, bgname, bgfilter)
            time.sleep(0.50)
            print(row)

## time
def iso(time):
    return time.isoformat() + ".00Z"

## execution scheme
try:
    with open('var.json') as file:
        logger.debug("read var.json")
except IOError as e:
    if whattodo.funct is 'define':
        logger.debug("var.json doesn't exist or not accessible trying to create a fresh var.json file now...")
    else:
        logger.info("var.json doesn't exist or not accessible please check permission or run './apex_bg_import.py -define'")
        exit(1)
if whattodo.funct is None:
    print ("Nothing to do")
    print ("apex_bg_import.py -h for help")
elif whattodo.funct is 'define':
    try:
        os.remove('var.json')
    except OSError:
        pass
    defvariables()
    monitoring()
    testvariables()
    print("***** to import Business Group list (no more than 125 subnets): ./apex_bg_import.py -import  *****")

elif 'import' in whattodo.funct:
    monitoring()
    sessionCookie = login()
    readcsv(readvar("csvpath","var.json"))
    logout(sessionCookie)

